@@ -3,8 +3,8 @@ use std::io::Write;
 use anyhow::{anyhow, bail, Context};
 use ge_man_lib::archive;
 use ge_man_lib::config::{LutrisConfig, SteamConfig};
-use ge_man_lib::download::{DownloadRequest, GeDownload};
 use ge_man_lib::download::response::DownloadedAssets;
+use ge_man_lib::download::{DownloadRequest, GeDownload};
 use ge_man_lib::error::{GithubError, LutrisConfigError, SteamConfigError};
 use ge_man_lib::tag::TagKind;
 use itertools::Itertools;
@@ -14,9 +14,16 @@ use crate::args::{
 };
 use crate::data::{ManagedVersion, ManagedVersions};
 use crate::filesystem::FilesystemManager;
-use crate::path::{AppConfigPaths, PathConfiguration, xdg_data_home};
+use crate::path::{xdg_data_home, AppConfigPaths, PathConfiguration};
 use crate::progress::{DownloadProgressTracker, ExtractionProgressTracker};
-use crate::version::Version;
+use crate::version::{Version, Versioned};
+
+const PROTON_APPLY_HINT: &str = "Successfully modified Steam config: If Steam is currently running, \
+any external change by GE-Man will not take effect and the new version can not be selected in the Steam settings!
+ 
+ To select the latest version you have two options:
+ \t1. Restart Steam to select the new version in Steam (which then requires a second restart for Steam to register the change).
+ \t2. Close Steam and run the apply command for your desired version. On the next start Steam will use the applied version.";
 
 trait AppConfig {
     fn version_dir_name(&self) -> String;
@@ -83,16 +90,16 @@ impl<'a> TerminalWriter<'a> {
     fn create_list_line(
         &self,
         version: ManagedVersion,
-        lutris_dir: Option<&String>,
-        steam_dir: Option<&String>,
+        wine_version_dir_name: Option<&String>,
+        proton_version_dir_name: Option<&String>,
     ) -> String {
-        if let Some(dir) = lutris_dir {
+        if let Some(dir) = wine_version_dir_name {
             if dir.eq(version.directory_name()) {
                 return format!("{} - In use by Lutris", version.tag());
             }
         }
 
-        if let Some(dir) = steam_dir {
+        if let Some(dir) = proton_version_dir_name {
             if dir.eq(version.directory_name()) {
                 return format!("{} - In use by Steam", version.tag());
             }
@@ -136,7 +143,7 @@ impl<'a> TerminalWriter<'a> {
         if !managed_versions.is_empty() {
             // Allow clone of version.kind() due to lifetime not living long enough.
             #[allow(clippy::clone_on_copy)]
-                let grouped_versions = managed_versions
+            let grouped_versions = managed_versions
                 .into_iter()
                 .sorted_unstable_by(|a, b| a.kind().cmp(b.kind()))
                 .group_by(|version| version.kind().clone());
@@ -144,7 +151,8 @@ impl<'a> TerminalWriter<'a> {
             for (kind, group) in &grouped_versions {
                 writeln!(stdout, "{}:", kind.compatibility_tool_name()).unwrap();
 
-                group.sorted_unstable_by(|a, b| a.tag().cmp_semver(b.tag()).reverse())
+                group
+                    .sorted_unstable_by(|a, b| a.tag().cmp_semver(b.tag()).reverse())
                     .for_each(|version| {
                         let line = self.create_list_line(version, wine_dir_name.as_ref(), proton_dir_name.as_ref());
                         writeln!(stdout, "* {}", line).unwrap();
@@ -163,15 +171,29 @@ impl<'a> TerminalWriter<'a> {
         let kind = args.tag_arg.kind;
         let mut managed_versions = self.read_managed_versions()?;
 
-        if tag.is_some() {
-            let version = Version::new(tag.cloned(), kind);
-            if managed_versions.find_version(&version).is_some() {
-                bail!("Given version is already managed");
+        let version = if tag.is_some() {
+            Version::new(tag.cloned(), kind)
+        } else {
+            match self.ge_downloader.fetch_release(tag.cloned(), kind) {
+                Ok(release) => Version::new(release.tag_name, kind),
+                Err(err) => {
+                    return Err(anyhow!(err).context(r#"Could not get latest tag for tagless "add" operation."#))
+                }
             }
+        };
+
+        if managed_versions.find_version(&version).is_some() {
+            writeln!(stdout, "Version {} is already managed", version)?;
+            return Ok(());
         }
 
         let download_tracker = Box::new(DownloadProgressTracker::default());
-        let request = DownloadRequest::new(tag.cloned(), kind, download_tracker, args.skip_checksum);
+        let request = DownloadRequest::new(
+            Some(version.tag().to_string()),
+            *version.kind(),
+            download_tracker,
+            args.skip_checksum,
+        );
 
         let assets = match self.ge_downloader.download_release_assets(request) {
             Ok(assets) => assets,
@@ -193,9 +215,9 @@ impl<'a> TerminalWriter<'a> {
         };
 
         let DownloadedAssets {
-            tag,
             compressed_archive: compressed_tar,
             checksum,
+            ..
         } = assets;
 
         if args.skip_checksum {
@@ -218,7 +240,6 @@ impl<'a> TerminalWriter<'a> {
             .inner()
             .wrap_read(std::io::Cursor::new(compressed_tar.compressed_content));
 
-        let version = Version::new(tag, kind);
         let version = self
             .fs_mng
             .setup_version(version, Box::new(compressed_tar_reader))
@@ -267,10 +288,12 @@ impl<'a> TerminalWriter<'a> {
                 match config {
                     Ok(config) => {
                         if self.check_if_version_in_use_by_config(&version, &config) {
-                            bail!("Wine version is in use by Lutris. Select a different version to make removal \
-                            possible.");
+                            bail!(
+                                "Wine version is in use by Lutris. Select a different version to make removal \
+                            possible."
+                            );
                         }
-                    },
+                    }
                     Err(err) => {
                         if let LutrisConfigError::IoError { source } = &err {
                             if source.raw_os_error().unwrap() != 2 {
@@ -291,8 +314,8 @@ impl<'a> TerminalWriter<'a> {
     }
 
     fn check_if_version_in_use_by_config<T>(&self, version: &ManagedVersion, app_config: &T) -> bool
-        where
-            T: AppConfig,
+    where
+        T: AppConfig,
     {
         app_config.version_dir_name().eq(version.directory_name())
     }
@@ -307,7 +330,7 @@ impl<'a> TerminalWriter<'a> {
                         kind.compatibility_tool_name(),
                         release.tag_name
                     )
-                        .unwrap();
+                    .unwrap();
                 }
                 Err(err) => {
                     writeln!(stderr, "Could not fetch latest release from Github: {}", err).unwrap();
@@ -327,7 +350,7 @@ impl<'a> TerminalWriter<'a> {
                         "Proton GE: Could not fetch release information from GitHub: {}",
                         err
                     )
-                        .unwrap(),
+                    .unwrap(),
                 }
 
                 match wine {
@@ -337,7 +360,7 @@ impl<'a> TerminalWriter<'a> {
                         "Wine GE: Could not fetch release information from GitHub: {}",
                         err
                     )
-                        .unwrap(),
+                    .unwrap(),
                 }
 
                 match lol {
@@ -347,7 +370,7 @@ impl<'a> TerminalWriter<'a> {
                         "Wine GE - LoL: Could not fetch release information from GitHub: {}",
                         err
                     )
-                        .unwrap(),
+                    .unwrap(),
                 }
             }
         }
@@ -375,19 +398,14 @@ impl<'a> TerminalWriter<'a> {
 
     fn do_apply_to_app_config(&self, stdout: &mut impl Write, version: &ManagedVersion) -> anyhow::Result<()> {
         let (modify_msg, success_msg) = match version.kind() {
-            TagKind::Proton => {
-                (
-                    format!("Modifying Steam configuration to use {}", version),
-                    "Successfully modified Steam config: If Steam is currently running, you need to select the \
-                    version in Steam. Otherwise, Steam will override the changes made by GE-Man."
-                )
-            }
-            TagKind::Wine { .. } => {
-                (
-                    format!("Modifying Lutris configuration to use {}", version),
-                    "Successfully modified Lutris config: Lutris should be restarted for the new settings to take effect."
-                )
-            }
+            TagKind::Proton => (
+                format!("Modifying Steam configuration to use {}", version),
+                PROTON_APPLY_HINT,
+            ),
+            TagKind::Wine { .. } => (
+                format!("Modifying Lutris configuration to use {}", version),
+                "Successfully modified Lutris config: Lutris should be restarted for the new settings to take effect.",
+            ),
         };
 
         writeln!(stdout, "{}", modify_msg).unwrap();
@@ -443,7 +461,7 @@ impl<'a> TerminalWriter<'a> {
             "Copied user_settings.py from {} to {}",
             src_version, dst_version
         )
-            .unwrap();
+        .unwrap();
         Ok(())
     }
 
@@ -462,8 +480,8 @@ impl<'a> TerminalWriter<'a> {
 
 #[cfg(test)]
 mod tests {
-    use std::{fs, io};
     use std::path::{Path, PathBuf};
+    use std::{fs, io};
 
     use anyhow::bail;
     use assert_fs::TempDir;
@@ -864,7 +882,7 @@ mod tests {
     }
 
     #[test]
-    fn add_existing_version_again_expect_error_msg() {
+    fn add_specific_version_which_is_already_managed_again_expect_message_about_already_being_managed() {
         let tag_arg = TagArg::new(Some(Tag::from("6.20-GE-1")), TagKind::Proton);
         let args = AddArgs::new(tag_arg, false, false);
 
@@ -886,11 +904,40 @@ mod tests {
 
         let mut stdout = AssertLines::new();
         let result = writer.add(&mut stdout, args);
-        assert!(result.is_err());
+        assert!(result.is_ok());
+        stdout.assert_line(0, "Version 6.20-GE-1 (Proton) is already managed");
+    }
 
-        let err = result.unwrap_err();
-        assert_eq!(err.to_string(), "Given version is already managed");
-        stdout.assert_empty();
+    #[test]
+    fn add_latest_version_which_is_already_managed_again_expect_message_about_already_being_managed() {
+        let tag_arg = TagArg::new(None, TagKind::Proton);
+        let args = AddArgs::new(tag_arg, false, false);
+
+        let mut fs_mng = MockFilesystemManager::new();
+        fs_mng.expect_setup_version().never();
+
+        let tmp_dir = TempDir::new().unwrap();
+        let json_path = tmp_dir.join("ge_man/managed_versions.json");
+        setup_managed_versions(&json_path, vec![ManagedVersion::new("6.20-GE-1", TagKind::Proton, "")]);
+
+        let mut path_cfg = MockPathConfiguration::new();
+        path_cfg
+            .expect_managed_versions_config()
+            .once()
+            .returning(move |_| json_path.clone());
+
+        let mut ge_downloader = MockDownloader::new();
+        ge_downloader
+            .expect_fetch_release()
+            .once()
+            .returning(move |_, _| Ok(GeRelease::new(String::from("6.20-GE-1"), Vec::new())));
+
+        let writer = TerminalWriter::new(&ge_downloader, &fs_mng, &path_cfg);
+
+        let mut stdout = AssertLines::new();
+        let result = writer.add(&mut stdout, args);
+        assert!(result.is_ok());
+        stdout.assert_line(0, "Version 6.20-GE-1 (Proton) is already managed");
     }
 
     #[test]
@@ -941,11 +988,7 @@ mod tests {
         stdout.assert_line(0, "Performing checksum comparison: Checksums match");
         stdout.assert_line(1, "Successfully added version");
         stdout.assert_line(2, "Modifying Steam configuration to use 6.20-GE-1 (Proton)");
-        stdout.assert_line(
-            3,
-            "Successfully modified Steam config: If Steam is currently running, you need to select the \
-                    version in Steam. Otherwise, Steam will override the changes made by GE-Man.",
-        );
+        stdout.assert_line(3, PROTON_APPLY_HINT);
     }
 
     #[test]
@@ -1295,11 +1338,7 @@ mod tests {
         writer.apply_to_app_config(&mut stdout, args).unwrap();
 
         stdout.assert_line(0, "Modifying Steam configuration to use 6.20-GE-1 (Proton)");
-        stdout.assert_line(
-            1,
-            "Successfully modified Steam config: If Steam is currently running, you need to select the \
-                    version in Steam. Otherwise, Steam will override the changes made by GE-Man.",
-        );
+        stdout.assert_line(1, PROTON_APPLY_HINT);
     }
 
     #[test]
@@ -1330,11 +1369,7 @@ mod tests {
         writer.apply_to_app_config(&mut stdout, args).unwrap();
 
         stdout.assert_line(0, "Modifying Steam configuration to use 6.20-GE-1 (Proton)");
-        stdout.assert_line(
-            1,
-            "Successfully modified Steam config: If Steam is currently running, you need to select the \
-                    version in Steam. Otherwise, Steam will override the changes made by GE-Man.",
-        );
+        stdout.assert_line(1, PROTON_APPLY_HINT);
     }
 
     #[test]
